import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { Disposable, MutableDisposable } from '../../../../base/common/lifecycle.js';
import { Event } from '../../../../base/common/event.js';
import Severity$1 from '../../../../base/common/severity.js';
import { localizeWithPath } from '../../../../nls.js';
import { IAccessibilityService } from '../../../../platform/accessibility/common/accessibility.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { NotificationPriority, INotificationService } from '../../../../platform/notification/common/notification.js';
import { IStatusbarService } from '../../../services/statusbar/browser/statusbar.js';
let AccessibilityStatus = class AccessibilityStatus extends Disposable {
    static { this.ID = 'workbench.contrib.accessibilityStatus'; }
    constructor(configurationService, notificationService, accessibilityService, statusbarService) {
        super();
        this.configurationService = configurationService;
        this.notificationService = notificationService;
        this.accessibilityService = accessibilityService;
        this.statusbarService = statusbarService;
        this.screenReaderNotification = null;
        this.promptedScreenReader = false;
        this.screenReaderModeElement = this._register(( new MutableDisposable()));
        this._register(CommandsRegistry.registerCommand({ id: 'showEditorScreenReaderNotification', handler: () => this.showScreenReaderNotification() }));
        this.updateScreenReaderModeElement(this.accessibilityService.isScreenReaderOptimized());
        this.registerListeners();
    }
    registerListeners() {
        this._register(this.accessibilityService.onDidChangeScreenReaderOptimized(() => this.onScreenReaderModeChange()));
        this._register(this.configurationService.onDidChangeConfiguration(c => {
            if (c.affectsConfiguration('editor.accessibilitySupport')) {
                this.onScreenReaderModeChange();
            }
        }));
    }
    showScreenReaderNotification() {
        this.screenReaderNotification = this.notificationService.prompt(Severity$1.Info, ( localizeWithPath(
            'vs/workbench/contrib/accessibility/browser/accessibilityStatus',
            'screenReaderDetectedExplanation.question',
            "Are you using a screen reader to operate VS Code?"
        )), [{
                label: ( localizeWithPath(
                    'vs/workbench/contrib/accessibility/browser/accessibilityStatus',
                    'screenReaderDetectedExplanation.answerYes',
                    "Yes"
                )),
                run: () => {
                    this.configurationService.updateValue('editor.accessibilitySupport', 'on', 2 );
                }
            }, {
                label: ( localizeWithPath(
                    'vs/workbench/contrib/accessibility/browser/accessibilityStatus',
                    'screenReaderDetectedExplanation.answerNo',
                    "No"
                )),
                run: () => {
                    this.configurationService.updateValue('editor.accessibilitySupport', 'off', 2 );
                }
            }], {
            sticky: true,
            priority: NotificationPriority.URGENT
        });
        Event.once(this.screenReaderNotification.onDidClose)(() => this.screenReaderNotification = null);
    }
    updateScreenReaderModeElement(visible) {
        if (visible) {
            if (!this.screenReaderModeElement.value) {
                const text = ( localizeWithPath(
                    'vs/workbench/contrib/accessibility/browser/accessibilityStatus',
                    'screenReaderDetected',
                    "Screen Reader Optimized"
                ));
                this.screenReaderModeElement.value = this.statusbarService.addEntry({
                    name: ( localizeWithPath(
                        'vs/workbench/contrib/accessibility/browser/accessibilityStatus',
                        'status.editor.screenReaderMode',
                        "Screen Reader Mode"
                    )),
                    text,
                    ariaLabel: text,
                    command: 'showEditorScreenReaderNotification',
                    kind: 'prominent',
                    showInAllWindows: true
                }, 'status.editor.screenReaderMode', 1 , 100.6);
            }
        }
        else {
            this.screenReaderModeElement.clear();
        }
    }
    onScreenReaderModeChange() {
        const screenReaderDetected = this.accessibilityService.isScreenReaderOptimized();
        if (screenReaderDetected) {
            const screenReaderConfiguration = this.configurationService.getValue('editor.accessibilitySupport');
            if (screenReaderConfiguration === 'auto') {
                if (!this.promptedScreenReader) {
                    this.promptedScreenReader = true;
                    setTimeout(() => this.showScreenReaderNotification(), 100);
                }
            }
        }
        if (this.screenReaderNotification) {
            this.screenReaderNotification.close();
        }
        this.updateScreenReaderModeElement(this.accessibilityService.isScreenReaderOptimized());
    }
};
AccessibilityStatus = ( __decorate([
    ( __param(0, IConfigurationService)),
    ( __param(1, INotificationService)),
    ( __param(2, IAccessibilityService)),
    ( __param(3, IStatusbarService))
], AccessibilityStatus));
export { AccessibilityStatus };
