import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { IExtensionsWorkbenchService } from '../common/extensions.js';
import { Severity, INotificationService } from '../../../../platform/notification/common/notification.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { localizeWithPath } from '../../../../nls.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { SearchExtensionsAction } from './extensionsActions.js';
import { distinct } from '../../../../base/common/arrays.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { IExtensionManagementService } from '../../../../platform/extensionManagement/common/extensionManagement.js';
import { areSameExtensions } from '../../../../platform/extensionManagement/common/extensionManagementUtil.js';
import { IWorkbenchExtensionEnablementService } from '../../../services/extensionManagement/common/extensionManagement.js';
let DeprecatedExtensionsChecker = class DeprecatedExtensionsChecker extends Disposable {
    constructor(extensionsWorkbenchService, extensionManagementService, extensionEnablementService, storageService, notificationService, instantiationService) {
        super();
        this.extensionsWorkbenchService = extensionsWorkbenchService;
        this.extensionEnablementService = extensionEnablementService;
        this.storageService = storageService;
        this.notificationService = notificationService;
        this.instantiationService = instantiationService;
        this.checkForDeprecatedExtensions();
        this._register(extensionManagementService.onDidInstallExtensions(e => {
            const ids = [];
            for (const { local } of e) {
                if (local && extensionsWorkbenchService.local.find(extension => areSameExtensions(extension.identifier, local.identifier))?.deprecationInfo) {
                    ids.push(local.identifier.id.toLowerCase());
                }
            }
            if (ids.length) {
                this.setNotifiedDeprecatedExtensions(ids);
            }
        }));
    }
    async checkForDeprecatedExtensions() {
        if (this.storageService.getBoolean('extensionsAssistant/doNotCheckDeprecated', 0 , false)) {
            return;
        }
        const local = await this.extensionsWorkbenchService.queryLocal();
        const previouslyNotified = this.getNotifiedDeprecatedExtensions();
        const toNotify = local.filter(e => !!e.deprecationInfo && e.local && this.extensionEnablementService.isEnabled(e.local)).filter(e => !previouslyNotified.includes(e.identifier.id.toLowerCase()));
        if (toNotify.length) {
            this.notificationService.prompt(Severity.Warning, ( localizeWithPath(
                'vs/workbench/contrib/extensions/browser/deprecatedExtensionsChecker',
                'deprecated extensions',
                "You have deprecated extensions installed. We recommend to review them and migrate to alternatives."
            )), [{
                    label: ( localizeWithPath(
                        'vs/workbench/contrib/extensions/browser/deprecatedExtensionsChecker',
                        'showDeprecated',
                        "Show Deprecated Extensions"
                    )),
                    run: async () => {
                        this.setNotifiedDeprecatedExtensions(( toNotify.map(e => e.identifier.id.toLowerCase())));
                        const action = this.instantiationService.createInstance(SearchExtensionsAction, ( toNotify.map(extension => `@id:${extension.identifier.id}`)).join(' '));
                        try {
                            await action.run();
                        }
                        finally {
                            action.dispose();
                        }
                    }
                }, {
                    label: ( localizeWithPath(
                        'vs/workbench/contrib/extensions/browser/deprecatedExtensionsChecker',
                        'neverShowAgain',
                        "Don't Show Again"
                    )),
                    isSecondary: true,
                    run: () => this.storageService.store('extensionsAssistant/doNotCheckDeprecated', true, 0 , 0 )
                }]);
        }
    }
    getNotifiedDeprecatedExtensions() {
        return JSON.parse(this.storageService.get('extensionsAssistant/deprecated', 0 , '[]'));
    }
    setNotifiedDeprecatedExtensions(notified) {
        this.storageService.store('extensionsAssistant/deprecated', JSON.stringify(distinct([...this.getNotifiedDeprecatedExtensions(), ...notified])), 0 , 0 );
    }
};
DeprecatedExtensionsChecker = ( __decorate([
    ( __param(0, IExtensionsWorkbenchService)),
    ( __param(1, IExtensionManagementService)),
    ( __param(2, IWorkbenchExtensionEnablementService)),
    ( __param(3, IStorageService)),
    ( __param(4, INotificationService)),
    ( __param(5, IInstantiationService))
], DeprecatedExtensionsChecker));
export { DeprecatedExtensionsChecker };
