import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { URI } from '../../../../base/common/uri.js';
import { MenuRegistry, MenuId } from '../../../../platform/actions/common/actions.js';
import { ITerminalService, ITerminalGroupService } from '../../terminal/browser/terminal.js';
import { ResourceContextKey } from '../../../common/contextkeys.js';
import { IFileService } from '../../../../platform/files/common/files.js';
import { IListService } from '../../../../platform/list/browser/listService.js';
import { getMultiSelectedResources, IExplorerService } from '../../files/browser/files.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { Schemas } from '../../../../base/common/network.js';
import { distinct } from '../../../../base/common/arrays.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IRemoteAgentService } from '../../../services/remote/common/remoteAgentService.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { Extensions } from '../../../common/contributions.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { isWindows } from '../../../../base/common/platform.js';
import { dirname, basename } from '../../../../base/common/path.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { IExternalTerminalService } from '../../../../platform/externalTerminal/common/externalTerminal.js';
import { TerminalLocation } from '../../../../platform/terminal/common/terminal.js';
const OPEN_IN_TERMINAL_COMMAND_ID = 'openInTerminal';
const OPEN_IN_INTEGRATED_TERMINAL_COMMAND_ID = 'openInIntegratedTerminal';
function registerOpenTerminalCommand(id, explorerKind) {
    CommandsRegistry.registerCommand({
        id: id,
        handler: async (accessor, resource) => {
            const configurationService = accessor.get(IConfigurationService);
            const editorService = accessor.get(IEditorService);
            const fileService = accessor.get(IFileService);
            const integratedTerminalService = accessor.get(ITerminalService);
            const remoteAgentService = accessor.get(IRemoteAgentService);
            const terminalGroupService = accessor.get(ITerminalGroupService);
            let externalTerminalService = undefined;
            try {
                externalTerminalService = accessor.get(IExternalTerminalService);
            }
            catch {
            }
            const resources = getMultiSelectedResources(resource, accessor.get(IListService), editorService, accessor.get(IExplorerService));
            return fileService.resolveAll(( resources.map(r => ({ resource: r })))).then(async (stats) => {
                const config = configurationService.getValue();
                const useIntegratedTerminal = remoteAgentService.getConnection() || explorerKind === 'integrated';
                const targets = distinct(stats.filter(data => data.success));
                if (useIntegratedTerminal) {
                    const opened = {};
                    const cwds = ( targets.map(({ stat }) => {
                        const resource = stat.resource;
                        if (stat.isDirectory) {
                            return resource;
                        }
                        return ( URI.from({
                            scheme: resource.scheme,
                            authority: resource.authority,
                            fragment: resource.fragment,
                            query: resource.query,
                            path: dirname(resource.path)
                        }));
                    }));
                    for (const cwd of cwds) {
                        if (opened[cwd.path]) {
                            return;
                        }
                        opened[cwd.path] = true;
                        const instance = await integratedTerminalService.createTerminal({ config: { cwd } });
                        if (instance && instance.target !== TerminalLocation.Editor && (resources.length === 1 || !resource || cwd.path === resource.path || cwd.path === dirname(resource.path))) {
                            integratedTerminalService.setActiveInstance(instance);
                            terminalGroupService.showPanel(true);
                        }
                    }
                }
                else if (externalTerminalService) {
                    distinct(( targets.map(
                        ({ stat }) => stat.isDirectory ? stat.resource.fsPath : dirname(stat.resource.fsPath)
                    ))).forEach(cwd => {
                        externalTerminalService.openTerminal(config.terminal.external, cwd);
                    });
                }
            });
        }
    });
}
registerOpenTerminalCommand(OPEN_IN_TERMINAL_COMMAND_ID, 'external');
registerOpenTerminalCommand(OPEN_IN_INTEGRATED_TERMINAL_COMMAND_ID, 'integrated');
let ExternalTerminalContribution = class ExternalTerminalContribution extends Disposable {
    constructor(_configurationService) {
        super();
        this._configurationService = _configurationService;
        const shouldShowIntegratedOnLocal = ( ContextKeyExpr.and(( ResourceContextKey.Scheme.isEqualTo(Schemas.file)), ( ContextKeyExpr.or(( ContextKeyExpr.equals('config.terminal.explorerKind', 'integrated')), ( ContextKeyExpr.equals('config.terminal.explorerKind', 'both'))))));
        const shouldShowExternalKindOnLocal = ( ContextKeyExpr.and(( ResourceContextKey.Scheme.isEqualTo(Schemas.file)), ( ContextKeyExpr.or(( ContextKeyExpr.equals('config.terminal.explorerKind', 'external')), ( ContextKeyExpr.equals('config.terminal.explorerKind', 'both'))))));
        this._openInIntegratedTerminalMenuItem = {
            group: 'navigation',
            order: 30,
            command: {
                id: OPEN_IN_INTEGRATED_TERMINAL_COMMAND_ID,
                title: ( localizeWithPath(
                    'vs/workbench/contrib/externalTerminal/browser/externalTerminal.contribution',
                    'scopedConsoleAction.Integrated',
                    "Open in Integrated Terminal"
                ))
            },
            when: ( ContextKeyExpr.or(shouldShowIntegratedOnLocal, ( ResourceContextKey.Scheme.isEqualTo(Schemas.vscodeRemote))))
        };
        this._openInTerminalMenuItem = {
            group: 'navigation',
            order: 31,
            command: {
                id: OPEN_IN_TERMINAL_COMMAND_ID,
                title: ( localizeWithPath(
                    'vs/workbench/contrib/externalTerminal/browser/externalTerminal.contribution',
                    'scopedConsoleAction.external',
                    "Open in External Terminal"
                ))
            },
            when: shouldShowExternalKindOnLocal
        };
        MenuRegistry.appendMenuItem(MenuId.ExplorerContext, this._openInTerminalMenuItem);
        MenuRegistry.appendMenuItem(MenuId.ExplorerContext, this._openInIntegratedTerminalMenuItem);
        this._configurationService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration('terminal.explorerKind') || e.affectsConfiguration('terminal.external')) {
                this._refreshOpenInTerminalMenuItemTitle();
            }
        });
        this._refreshOpenInTerminalMenuItemTitle();
    }
    isWindows() {
        const config = this._configurationService.getValue().terminal;
        if (isWindows && config.external?.windowsExec) {
            const file = basename(config.external.windowsExec);
            if (file === 'wt' || file === 'wt.exe') {
                return true;
            }
        }
        return false;
    }
    _refreshOpenInTerminalMenuItemTitle() {
        if (this.isWindows()) {
            this._openInTerminalMenuItem.command.title = ( localizeWithPath(
                'vs/workbench/contrib/externalTerminal/browser/externalTerminal.contribution',
                'scopedConsoleAction.wt',
                "Open in Windows Terminal"
            ));
        }
    }
};
ExternalTerminalContribution = ( __decorate([
    ( __param(0, IConfigurationService))
], ExternalTerminalContribution));
( Registry.as(Extensions.Workbench)).registerWorkbenchContribution(ExternalTerminalContribution, 3 );
export { ExternalTerminalContribution };
