import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { IPreferencesService } from '../../../services/preferences/common/preferences.js';
import { settingKeyToDisplayFormat } from '../../preferences/browser/settingsTreeModels.js';
import { Schemas } from '../../../../base/common/network.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { DefaultSettings } from '../../../services/preferences/common/preferencesModels.js';
import { IContextMenuService } from '../../../../platform/contextview/browser/contextView.js';
import { ActionViewItem } from '../../../../base/browser/ui/actionbar/actionViewItems.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
import { IClipboardService } from '../../../../platform/clipboard/common/clipboardService.js';
const codeSettingRegex = /^<code (codesetting)="([^\s"\:]+)(?::([^"]+))?">/;
let SimpleSettingRenderer = class SimpleSettingRenderer {
    constructor(_configurationService, _contextMenuService, _preferencesService, _telemetryService, _clipboardService) {
        this._configurationService = _configurationService;
        this._contextMenuService = _contextMenuService;
        this._preferencesService = _preferencesService;
        this._telemetryService = _telemetryService;
        this._clipboardService = _clipboardService;
        this._updatedSettings = ( new Map());
        this._encounteredSettings = ( new Map());
        this._featuredSettings = ( new Map());
        this.settingsGroups = undefined;
        this._defaultSettings = ( new DefaultSettings([], 2 ));
    }
    get featuredSettingStates() {
        const result = ( new Map());
        for (const [settingId, value] of this._featuredSettings) {
            result.set(settingId, this._configurationService.getValue(settingId) === value);
        }
        return result;
    }
    getHtmlRenderer() {
        return (html) => {
            const match = codeSettingRegex.exec(html);
            if (match && match.length === 4) {
                const settingId = match[2];
                const rendered = this.render(settingId, match[3]);
                if (rendered) {
                    html = html.replace(codeSettingRegex, rendered);
                }
            }
            return html;
        };
    }
    settingToUriString(settingId, value) {
        return `${Schemas.codeSetting}://${settingId}${value ? `/${value}` : ''}`;
    }
    getSetting(settingId) {
        if (!this.settingsGroups) {
            this.settingsGroups = this._defaultSettings.getSettingsGroups();
        }
        if (( this._encounteredSettings.has(settingId))) {
            return this._encounteredSettings.get(settingId);
        }
        for (const group of this.settingsGroups) {
            for (const section of group.sections) {
                for (const setting of section.settings) {
                    if (setting.key === settingId) {
                        this._encounteredSettings.set(settingId, setting);
                        return setting;
                    }
                }
            }
        }
        return undefined;
    }
    parseValue(settingId, value) {
        if (value === 'undefined' || value === '') {
            return undefined;
        }
        const setting = this.getSetting(settingId);
        if (!setting) {
            return value;
        }
        switch (setting.type) {
            case 'boolean':
                return value === 'true';
            case 'number':
                return parseInt(value, 10);
            case 'string':
            default:
                return value;
        }
    }
    render(settingId, newValue) {
        const setting = this.getSetting(settingId);
        if (!setting) {
            return '';
        }
        return this.renderSetting(setting, newValue);
    }
    viewInSettingsMessage(settingId, alreadyDisplayed) {
        if (alreadyDisplayed) {
            return ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'viewInSettings',
                "View in Settings"
            ));
        }
        else {
            const displayName = settingKeyToDisplayFormat(settingId);
            return ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'viewInSettingsDetailed',
                "View \"{0}: {1}\" in Settings",
                displayName.category,
                displayName.label
            ));
        }
    }
    restorePreviousSettingMessage(settingId) {
        const displayName = settingKeyToDisplayFormat(settingId);
        return ( localizeWithPath(
            'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
            'restorePreviousValue',
            "Restore value of \"{0}: {1}\"",
            displayName.category,
            displayName.label
        ));
    }
    booleanSettingMessage(setting, booleanValue) {
        const currentValue = this._configurationService.getValue(setting.key);
        if (currentValue === booleanValue || (currentValue === undefined && setting.value === booleanValue)) {
            return undefined;
        }
        const displayName = settingKeyToDisplayFormat(setting.key);
        if (booleanValue) {
            return ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'trueMessage',
                "Enable \"{0}: {1}\"",
                displayName.category,
                displayName.label
            ));
        }
        else {
            return ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'falseMessage',
                "Disable \"{0}: {1}\"",
                displayName.category,
                displayName.label
            ));
        }
    }
    stringSettingMessage(setting, stringValue) {
        const currentValue = this._configurationService.getValue(setting.key);
        if (currentValue === stringValue || (currentValue === undefined && setting.value === stringValue)) {
            return undefined;
        }
        const displayName = settingKeyToDisplayFormat(setting.key);
        return ( localizeWithPath(
            'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
            'stringValue',
            "Set \"{0}: {1}\" to \"{2}\"",
            displayName.category,
            displayName.label,
            stringValue
        ));
    }
    numberSettingMessage(setting, numberValue) {
        const currentValue = this._configurationService.getValue(setting.key);
        if (currentValue === numberValue || (currentValue === undefined && setting.value === numberValue)) {
            return undefined;
        }
        const displayName = settingKeyToDisplayFormat(setting.key);
        return ( localizeWithPath(
            'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
            'numberValue',
            "Set \"{0}: {1}\" to {2}",
            displayName.category,
            displayName.label,
            numberValue
        ));
    }
    renderSetting(setting, newValue) {
        const href = this.settingToUriString(setting.key, newValue);
        const title = ( localizeWithPath(
            'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
            'changeSettingTitle',
            "View or change setting"
        ));
        return `<code tabindex="0"><a href="${href}" class="codesetting" title="${title}" aria-role="button"><svg width="14" height="14" viewBox="0 0 15 15" xmlns="http://www.w3.org/2000/svg" fill="currentColor"><path d="M9.1 4.4L8.6 2H7.4l-.5 2.4-.7.3-2-1.3-.9.8 1.3 2-.2.7-2.4.5v1.2l2.4.5.3.8-1.3 2 .8.8 2-1.3.8.3.4 2.3h1.2l.5-2.4.8-.3 2 1.3.8-.8-1.3-2 .3-.8 2.3-.4V7.4l-2.4-.5-.3-.8 1.3-2-.8-.8-2 1.3-.7-.2zM9.4 1l.5 2.4L12 2.1l2 2-1.4 2.1 2.4.4v2.8l-2.4.5L14 12l-2 2-2.1-1.4-.5 2.4H6.6l-.5-2.4L4 13.9l-2-2 1.4-2.1L1 9.4V6.6l2.4-.5L2.1 4l2-2 2.1 1.4.4-2.4h2.8zm.6 7c0 1.1-.9 2-2 2s-2-.9-2-2 .9-2 2-2 2 .9 2 2zM8 9c.6 0 1-.4 1-1s-.4-1-1-1-1 .4-1 1 .4 1 1 1z"/></svg>
			<span class="separator"></span>
			<span class="setting-name">${setting.key}</span>
		</a></code><code>`;
    }
    getSettingMessage(setting, newValue) {
        if (setting.type === 'boolean') {
            return this.booleanSettingMessage(setting, newValue);
        }
        else if (setting.type === 'string') {
            return this.stringSettingMessage(setting, newValue);
        }
        else if (setting.type === 'number') {
            return this.numberSettingMessage(setting, newValue);
        }
        return undefined;
    }
    async restoreSetting(settingId) {
        const userOriginalSettingValue = this._updatedSettings.get(settingId);
        this._updatedSettings.delete(settingId);
        return this._configurationService.updateValue(settingId, userOriginalSettingValue, 2 );
    }
    async setSetting(settingId, currentSettingValue, newSettingValue) {
        this._updatedSettings.set(settingId, currentSettingValue);
        return this._configurationService.updateValue(settingId, newSettingValue, 2 );
    }
    getActions(uri) {
        if (uri.scheme !== Schemas.codeSetting) {
            return;
        }
        const actions = [];
        const settingId = uri.authority;
        const newSettingValue = this.parseValue(uri.authority, uri.path.substring(1));
        const currentSettingValue = this._configurationService.inspect(settingId).userValue;
        if ((newSettingValue !== undefined) && newSettingValue === currentSettingValue && ( this._updatedSettings.has(settingId))) {
            const restoreMessage = this.restorePreviousSettingMessage(settingId);
            actions.push({
                class: undefined,
                id: 'restoreSetting',
                enabled: true,
                tooltip: restoreMessage,
                label: restoreMessage,
                run: () => {
                    return this.restoreSetting(settingId);
                }
            });
        }
        else if (newSettingValue !== undefined) {
            const setting = this.getSetting(settingId);
            const trySettingMessage = setting ? this.getSettingMessage(setting, newSettingValue) : undefined;
            if (setting && trySettingMessage) {
                actions.push({
                    class: undefined,
                    id: 'trySetting',
                    enabled: currentSettingValue !== newSettingValue,
                    tooltip: trySettingMessage,
                    label: trySettingMessage,
                    run: () => {
                        this.setSetting(settingId, currentSettingValue, newSettingValue);
                    }
                });
            }
        }
        const viewInSettingsMessage = this.viewInSettingsMessage(settingId, actions.length > 0);
        actions.push({
            class: undefined,
            enabled: true,
            id: 'viewInSettings',
            tooltip: viewInSettingsMessage,
            label: viewInSettingsMessage,
            run: () => {
                return this._preferencesService.openApplicationSettings({ query: `@id:${settingId}` });
            }
        });
        actions.push({
            class: undefined,
            enabled: true,
            id: 'copySettingId',
            tooltip: ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'copySettingId',
                "Copy Setting ID"
            )),
            label: ( localizeWithPath(
                'vs/workbench/contrib/markdown/browser/markdownSettingRenderer',
                'copySettingId',
                "Copy Setting ID"
            )),
            run: () => {
                this._clipboardService.writeText(settingId);
            }
        });
        return actions;
    }
    showContextMenu(uri, x, y) {
        const actions = this.getActions(uri);
        if (!actions) {
            return;
        }
        this._contextMenuService.showContextMenu({
            getAnchor: () => ({ x, y }),
            getActions: () => actions,
            getActionViewItem: (action) => {
                return ( new ActionViewItem(action, action, { label: true }));
            },
        });
    }
    async updateSetting(uri, x, y) {
        if (uri.scheme === Schemas.codeSetting) {
            this._telemetryService.publicLog2('releaseNotesSettingAction', {
                settingId: uri.authority
            });
            return this.showContextMenu(uri, x, y);
        }
    }
};
SimpleSettingRenderer = ( __decorate([
    ( __param(0, IConfigurationService)),
    ( __param(1, IContextMenuService)),
    ( __param(2, IPreferencesService)),
    ( __param(3, ITelemetryService)),
    ( __param(4, IClipboardService))
], SimpleSettingRenderer));
export { SimpleSettingRenderer };
