import { localizeWithPath } from '../../../../nls.js';
import { format } from '../../../../base/common/strings.js';
import { IKeybindingService } from '../../../../platform/keybinding/common/keybinding.js';
import { IAccessibleViewService } from '../../accessibility/browser/accessibleView.js';
import { getNotebookEditorFromEditorPane } from './notebookBrowser.js';
function getAccessibilityHelpText(accessor) {
    const keybindingService = accessor.get(IKeybindingService);
    const content = [];
    content.push(( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.overview',
        'The notebook view is a collection of code and markdown cells. Code cells can be executed and will produce output directly below the cell.'
    )));
    content.push(descriptionForCommand('notebook.cell.edit', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.edit',
        'The Edit Cell command ({0}) will focus on the cell input.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.editNoKb',
        'The Edit Cell command will focus on the cell input and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(descriptionForCommand('notebook.cell.quitEdit', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.quitEdit',
        'The Quit Edit command ({0}) will set focus on the cell container. The default (Escape) key may need to be pressed twice first exit the virtual cursor if active.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.quitEditNoKb',
        'The Quit Edit command will set focus on the cell container and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(descriptionForCommand('notebook.cell.focusInOutput', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.focusInOutput',
        'The Focus Output command ({0}) will set focus in the cell\'s output.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.focusInOutputNoKb',
        'The Quit Edit command will set focus in the cell\'s output and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(descriptionForCommand('notebook.focusNextEditor', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.focusNextEditor',
        'The Focus Next Cell Editor command ({0}) will set focus in the next cell\'s editor.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.focusNextEditorNoKb',
        'The Focus Next Cell Editor command will set focus in the next cell\'s editor and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(descriptionForCommand('notebook.focusPreviousEditor', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.focusPreviousEditor',
        'The Focus Previous Cell Editor command ({0}) will set focus in the previous cell\'s editor.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.focusPreviousEditorNoKb',
        'The Focus Previous Cell Editor command will set focus in the previous cell\'s editor and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cellNavigation',
        'The up and down arrows will also move focus between cells while focused on the outer cell container.'
    )));
    content.push(descriptionForCommand('notebook.cell.executeAndFocusContainer', ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.executeAndFocusContainer',
        'The Execute Cell command ({0}) executes the cell that currently has focus.'
    )), ( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.executeAndFocusContainerNoKb',
        'The Execute Cell command executes the cell that currently has focus and is currently not triggerable by a keybinding.'
    )), keybindingService));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.cell.insertCodeCellBelowAndFocusContainer',
        'The Insert Cell Above/Below commands will create new empty code cells'
    )));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/notebook/browser/notebookAccessibility',
        'notebook.changeCellType',
        'The Change Cell to Code/Markdown commands are used to switch between cell types.'
    )));
    return content.join('\n\n');
}
function descriptionForCommand(commandId, msg, noKbMsg, keybindingService) {
    const kb = keybindingService.lookupKeybinding(commandId);
    if (kb) {
        return format(msg, kb.getAriaLabel());
    }
    return format(noKbMsg, commandId);
}
async function runAccessibilityHelpAction(accessor, editor) {
    const accessibleViewService = accessor.get(IAccessibleViewService);
    const helpText = getAccessibilityHelpText(accessor);
    accessibleViewService.show({
        id: "notebook" ,
        verbositySettingKey: "accessibility.verbosity.notebook" ,
        provideContent: () => helpText,
        onClose: () => {
            editor.focus();
        },
        options: { type: "help"  }
    });
}
function showAccessibleOutput(accessibleViewService, editorService) {
    const activePane = editorService.activeEditorPane;
    const notebookEditor = getNotebookEditorFromEditorPane(activePane);
    const notebookViewModel = notebookEditor?.getViewModel();
    const selections = notebookViewModel?.getSelections();
    const notebookDocument = notebookViewModel?.notebookDocument;
    if (!selections || !notebookDocument || !notebookEditor?.textModel) {
        return false;
    }
    const viewCell = notebookViewModel.viewCells[selections[0].start];
    let outputContent = '';
    const decoder = ( new TextDecoder());
    for (let i = 0; i < viewCell.outputsViewModels.length; i++) {
        const outputViewModel = viewCell.outputsViewModels[i];
        const outputTextModel = viewCell.model.outputs[i];
        const [mimeTypes, pick] = outputViewModel.resolveMimeTypes(notebookEditor.textModel, undefined);
        const mimeType = mimeTypes[pick].mimeType;
        let buffer = outputTextModel.outputs.find(output => output.mime === mimeType);
        if (!buffer || mimeType.startsWith('image')) {
            buffer = outputTextModel.outputs.find(output => !output.mime.startsWith('image'));
        }
        let text = `${mimeType}`;
        if (buffer) {
            const charLimit = 100_000;
            text = decoder.decode(buffer.data.slice(0, charLimit).buffer);
            if (buffer.data.byteLength > charLimit) {
                text = text + '...(truncated)';
            }
            if (mimeType.endsWith('error')) {
                text = text.replace(/\\u001b\[[0-9;]*m/gi, '').replaceAll('\\n', '\n');
            }
        }
        const index = viewCell.outputsViewModels.length > 1
            ? `Cell output ${i + 1} of ${viewCell.outputsViewModels.length}\n`
            : '';
        outputContent = outputContent.concat(`${index}${text}\n`);
    }
    if (!outputContent) {
        return false;
    }
    accessibleViewService.show({
        id: "notebook" ,
        verbositySettingKey: "accessibility.verbosity.notebook" ,
        provideContent() { return outputContent; },
        onClose() {
            notebookEditor?.setFocus(selections[0]);
            activePane?.focus();
        },
        options: { type: "view"  }
    });
    return true;
}
export { getAccessibilityHelpText, runAccessibilityHelpAction, showAccessibleOutput };
