import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { basename } from '../../../../base/common/resources.js';
import { Disposable, MutableDisposable, DisposableStore, combinedDisposable, dispose } from '../../../../base/common/lifecycle.js';
import { Event } from '../../../../base/common/event.js';
import { VIEW_PANE_ID, ISCMService, ISCMViewService } from '../common/scm.js';
import { NumberBadge, IActivityService } from '../../../services/activity/common/activity.js';
import { RawContextKey, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { IStatusbarService } from '../../../services/statusbar/browser/statusbar.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { EditorResourceAccessor } from '../../../common/editor.js';
import { IUriIdentityService } from '../../../../platform/uriIdentity/common/uriIdentity.js';
import { Schemas } from '../../../../base/common/network.js';
import { Iterable } from '../../../../base/common/iterator.js';
import { ITitleService } from '../../../services/title/browser/titleService.js';
function getCount(repository) {
    if (typeof repository.provider.count === 'number') {
        return repository.provider.count;
    }
    else {
        return repository.provider.groups.reduce((r, g) => r + g.resources.length, 0);
    }
}
let SCMStatusController = class SCMStatusController {
    constructor(scmService, scmViewService, statusbarService, activityService, editorService, configurationService, uriIdentityService) {
        this.scmService = scmService;
        this.scmViewService = scmViewService;
        this.statusbarService = statusbarService;
        this.activityService = activityService;
        this.editorService = editorService;
        this.configurationService = configurationService;
        this.uriIdentityService = uriIdentityService;
        this.statusBarDisposable = Disposable.None;
        this.focusDisposable = Disposable.None;
        this.focusedRepository = undefined;
        this.badgeDisposable = ( new MutableDisposable());
        this.disposables = ( new DisposableStore());
        this.repositoryDisposables = ( new Set());
        this.scmService.onDidAddRepository(this.onDidAddRepository, this, this.disposables);
        this.scmService.onDidRemoveRepository(this.onDidRemoveRepository, this, this.disposables);
        const onDidChangeSCMCountBadge = Event.filter(configurationService.onDidChangeConfiguration, e => e.affectsConfiguration('scm.countBadge'));
        onDidChangeSCMCountBadge(this.renderActivityCount, this, this.disposables);
        for (const repository of this.scmService.repositories) {
            this.onDidAddRepository(repository);
        }
        this.scmViewService.onDidFocusRepository(this.focusRepository, this, this.disposables);
        this.focusRepository(this.scmViewService.focusedRepository);
        editorService.onDidActiveEditorChange(() => this.tryFocusRepositoryBasedOnActiveEditor(), this, this.disposables);
        this.renderActivityCount();
    }
    tryFocusRepositoryBasedOnActiveEditor(repositories = this.scmService.repositories) {
        const resource = EditorResourceAccessor.getOriginalUri(this.editorService.activeEditor);
        if (!resource) {
            return false;
        }
        let bestRepository = null;
        let bestMatchLength = Number.POSITIVE_INFINITY;
        for (const repository of repositories) {
            const root = repository.provider.rootUri;
            if (!root) {
                continue;
            }
            const path = this.uriIdentityService.extUri.relativePath(root, resource);
            if (path && !/^\.\./.test(path) && path.length < bestMatchLength) {
                bestRepository = repository;
                bestMatchLength = path.length;
            }
        }
        if (!bestRepository) {
            return false;
        }
        this.focusRepository(bestRepository);
        return true;
    }
    onDidAddRepository(repository) {
        const onDidChange = Event.any(repository.provider.onDidChange, repository.provider.onDidChangeResources);
        const changeDisposable = onDidChange(() => this.renderActivityCount());
        const onDidRemove = Event.filter(this.scmService.onDidRemoveRepository, e => e === repository);
        const removeDisposable = onDidRemove(() => {
            disposable.dispose();
            this.repositoryDisposables.delete(disposable);
            this.renderActivityCount();
        });
        const disposable = combinedDisposable(changeDisposable, removeDisposable);
        this.repositoryDisposables.add(disposable);
        this.tryFocusRepositoryBasedOnActiveEditor(Iterable.single(repository));
    }
    onDidRemoveRepository(repository) {
        if (this.focusedRepository !== repository) {
            return;
        }
        this.focusRepository(Iterable.first(this.scmService.repositories));
    }
    focusRepository(repository) {
        if (this.focusedRepository === repository) {
            return;
        }
        this.focusDisposable.dispose();
        this.focusedRepository = repository;
        if (repository && repository.provider.onDidChangeStatusBarCommands) {
            this.focusDisposable = repository.provider.onDidChangeStatusBarCommands(() => this.renderStatusBar(repository));
        }
        this.renderStatusBar(repository);
        this.renderActivityCount();
    }
    renderStatusBar(repository) {
        this.statusBarDisposable.dispose();
        if (!repository) {
            return;
        }
        const commands = repository.provider.statusBarCommands || [];
        const label = repository.provider.rootUri
            ? `${basename(repository.provider.rootUri)} (${repository.provider.label})`
            : repository.provider.label;
        const disposables = ( new DisposableStore());
        for (let index = 0; index < commands.length; index++) {
            const command = commands[index];
            const tooltip = `${label}${command.tooltip ? ` - ${command.tooltip}` : ''}`;
            let repoAgnosticActionName = command.arguments?.[0];
            if (repoAgnosticActionName && typeof repoAgnosticActionName === 'string') {
                repoAgnosticActionName = repoAgnosticActionName
                    .substring(0, repoAgnosticActionName.lastIndexOf('/'))
                    .replace(/^git\./, '');
                if (repoAgnosticActionName.length > 1) {
                    repoAgnosticActionName = repoAgnosticActionName[0].toLocaleUpperCase() + repoAgnosticActionName.slice(1);
                }
            }
            else {
                repoAgnosticActionName = '';
            }
            const statusbarEntry = {
                name: ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/activity',
                    'status.scm',
                    "Source Control"
                )) + (repoAgnosticActionName ? ` ${repoAgnosticActionName}` : ''),
                text: command.title,
                ariaLabel: tooltip,
                tooltip,
                command: command.id ? command : undefined
            };
            disposables.add(index === 0 ?
                this.statusbarService.addEntry(statusbarEntry, `status.scm.${index}`, 0 , 10000) :
                this.statusbarService.addEntry(statusbarEntry, `status.scm.${index}`, 0 , { id: `status.scm.${index - 1}`, alignment: 1 , compact: true }));
        }
        this.statusBarDisposable = disposables;
    }
    renderActivityCount() {
        const countBadgeType = this.configurationService.getValue('scm.countBadge');
        let count = 0;
        if (countBadgeType === 'all') {
            count = Iterable.reduce(this.scmService.repositories, (r, repository) => r + getCount(repository), 0);
        }
        else if (countBadgeType === 'focused' && this.focusedRepository) {
            count = getCount(this.focusedRepository);
        }
        if (count > 0) {
            const badge = ( new NumberBadge(count, num => ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/activity',
                'scmPendingChangesBadge',
                '{0} pending changes',
                num
            ))));
            this.badgeDisposable.value = this.activityService.showViewActivity(VIEW_PANE_ID, { badge });
        }
        else {
            this.badgeDisposable.value = undefined;
        }
    }
    dispose() {
        this.focusDisposable.dispose();
        this.statusBarDisposable.dispose();
        this.badgeDisposable.dispose();
        this.disposables.dispose();
        dispose(( this.repositoryDisposables.values()));
        this.repositoryDisposables.clear();
    }
};
SCMStatusController = ( __decorate([
    ( __param(0, ISCMService)),
    ( __param(1, ISCMViewService)),
    ( __param(2, IStatusbarService)),
    ( __param(3, IActivityService)),
    ( __param(4, IEditorService)),
    ( __param(5, IConfigurationService)),
    ( __param(6, IUriIdentityService))
], SCMStatusController));
const ActiveRepositoryContextKeys = {
    ActiveRepositoryName: ( new RawContextKey('scmActiveRepositoryName', '')),
    ActiveRepositoryBranchName: ( new RawContextKey('scmActiveRepositoryBranchName', '')),
};
let SCMActiveRepositoryContextKeyController = class SCMActiveRepositoryContextKeyController {
    constructor(contextKeyService, editorService, scmViewService, titleService, uriIdentityService) {
        this.editorService = editorService;
        this.scmViewService = scmViewService;
        this.uriIdentityService = uriIdentityService;
        this.focusedRepository = undefined;
        this.focusDisposable = Disposable.None;
        this.disposables = ( new DisposableStore());
        this.activeRepositoryNameContextKey = ActiveRepositoryContextKeys.ActiveRepositoryName.bindTo(contextKeyService);
        this.activeRepositoryBranchNameContextKey = ActiveRepositoryContextKeys.ActiveRepositoryBranchName.bindTo(contextKeyService);
        titleService.registerVariables([
            { name: 'activeRepositoryName', contextKey: ActiveRepositoryContextKeys.ActiveRepositoryName.key },
            { name: 'activeRepositoryBranchName', contextKey: ActiveRepositoryContextKeys.ActiveRepositoryBranchName.key, }
        ]);
        editorService.onDidActiveEditorChange(this.onDidActiveEditorChange, this, this.disposables);
        scmViewService.onDidFocusRepository(this.onDidFocusRepository, this, this.disposables);
        this.onDidFocusRepository(scmViewService.focusedRepository);
    }
    onDidActiveEditorChange() {
        const activeResource = EditorResourceAccessor.getOriginalUri(this.editorService.activeEditor);
        if (activeResource?.scheme !== Schemas.file && activeResource?.scheme !== Schemas.vscodeRemote) {
            return;
        }
        const repository = Iterable.find(this.scmViewService.repositories, r => Boolean(r.provider.rootUri && this.uriIdentityService.extUri.isEqualOrParent(activeResource, r.provider.rootUri)));
        this.onDidFocusRepository(repository);
    }
    onDidFocusRepository(repository) {
        if (!repository || this.focusedRepository === repository) {
            return;
        }
        this.focusDisposable.dispose();
        this.focusedRepository = repository;
        if (repository && repository.provider.onDidChangeStatusBarCommands) {
            this.focusDisposable = repository.provider.onDidChangeStatusBarCommands(() => this.updateContextKeys(repository));
        }
        this.updateContextKeys(repository);
    }
    updateContextKeys(repository) {
        this.activeRepositoryNameContextKey.set(repository?.provider.name ?? '');
        this.activeRepositoryBranchNameContextKey.set(repository?.provider.historyProvider?.currentHistoryItemGroup?.label ?? '');
    }
    dispose() {
        this.focusDisposable.dispose();
        this.disposables.dispose();
    }
};
SCMActiveRepositoryContextKeyController = ( __decorate([
    ( __param(0, IContextKeyService)),
    ( __param(1, IEditorService)),
    ( __param(2, ISCMViewService)),
    ( __param(3, ITitleService)),
    ( __param(4, IUriIdentityService))
], SCMActiveRepositoryContextKeyController));
let SCMActiveResourceContextKeyController = class SCMActiveResourceContextKeyController {
    constructor(contextKeyService, editorService, scmService, uriIdentityService) {
        this.editorService = editorService;
        this.scmService = scmService;
        this.uriIdentityService = uriIdentityService;
        this.disposables = ( new DisposableStore());
        this.repositoryDisposables = ( new Set());
        this.activeResourceHasChangesContextKey = contextKeyService.createKey('scmActiveResourceHasChanges', false);
        this.activeResourceRepositoryContextKey = contextKeyService.createKey('scmActiveResourceRepository', undefined);
        this.scmService.onDidAddRepository(this.onDidAddRepository, this, this.disposables);
        for (const repository of this.scmService.repositories) {
            this.onDidAddRepository(repository);
        }
        editorService.onDidActiveEditorChange(this.updateContextKey, this, this.disposables);
    }
    onDidAddRepository(repository) {
        const onDidChange = Event.any(repository.provider.onDidChange, repository.provider.onDidChangeResources);
        const changeDisposable = onDidChange(() => this.updateContextKey());
        const onDidRemove = Event.filter(this.scmService.onDidRemoveRepository, e => e === repository);
        const removeDisposable = onDidRemove(() => {
            disposable.dispose();
            this.repositoryDisposables.delete(disposable);
            this.updateContextKey();
        });
        const disposable = combinedDisposable(changeDisposable, removeDisposable);
        this.repositoryDisposables.add(disposable);
    }
    updateContextKey() {
        const activeResource = EditorResourceAccessor.getOriginalUri(this.editorService.activeEditor);
        if (activeResource?.scheme === Schemas.file || activeResource?.scheme === Schemas.vscodeRemote) {
            const activeResourceRepository = Iterable.find(this.scmService.repositories, r => Boolean(r.provider.rootUri && this.uriIdentityService.extUri.isEqualOrParent(activeResource, r.provider.rootUri)));
            this.activeResourceRepositoryContextKey.set(activeResourceRepository?.id);
            for (const resourceGroup of activeResourceRepository?.provider.groups ?? []) {
                if (( resourceGroup.resources
                    .some(
                    scmResource => this.uriIdentityService.extUri.isEqual(activeResource, scmResource.sourceUri)
                ))) {
                    this.activeResourceHasChangesContextKey.set(true);
                    return;
                }
            }
            this.activeResourceHasChangesContextKey.set(false);
        }
        else {
            this.activeResourceHasChangesContextKey.set(false);
            this.activeResourceRepositoryContextKey.set(undefined);
        }
    }
    dispose() {
        this.disposables.dispose();
        dispose(( this.repositoryDisposables.values()));
        this.repositoryDisposables.clear();
    }
};
SCMActiveResourceContextKeyController = ( __decorate([
    ( __param(0, IContextKeyService)),
    ( __param(1, IEditorService)),
    ( __param(2, ISCMService)),
    ( __param(3, IUriIdentityService))
], SCMActiveResourceContextKeyController));
export { SCMActiveRepositoryContextKeyController, SCMActiveResourceContextKeyController, SCMStatusController };
