import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { language } from '../../../../base/common/platform.js';
import { Extensions } from '../../../common/contributions.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { Severity, INotificationService } from '../../../../platform/notification/common/notification.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { IWorkbenchAssignmentService } from '../../../services/assignment/common/assignmentService.js';
import { URI } from '../../../../base/common/uri.js';
import { platform } from '../../../../base/common/process.js';
import { ThrottledDelayer } from '../../../../base/common/async.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { Event } from '../../../../base/common/event.js';
const WAIT_TIME_TO_SHOW_SURVEY = 1000 * 60 * 60;
const MIN_WAIT_TIME_TO_SHOW_SURVEY = 1000 * 60 * 2;
const MAX_INSTALL_AGE = 1000 * 60 * 60 * 24;
const REMIND_LATER_DELAY = 1000 * 60 * 60 * 4;
const SKIP_SURVEY_KEY = 'ces/skipSurvey';
const REMIND_LATER_DATE_KEY = 'ces/remindLaterDate';
let CESContribution = class CESContribution extends Disposable {
    constructor(storageService, notificationService, telemetryService, openerService, productService, tasExperimentService) {
        super();
        this.storageService = storageService;
        this.notificationService = notificationService;
        this.telemetryService = telemetryService;
        this.openerService = openerService;
        this.productService = productService;
        this.promptDelayer = this._register(( new ThrottledDelayer(0)));
        this.tasExperimentService = tasExperimentService;
        if (!productService.cesSurveyUrl) {
            return;
        }
        const skipSurvey = storageService.get(SKIP_SURVEY_KEY, -1 , '');
        if (skipSurvey) {
            return;
        }
        this.schedulePrompt();
    }
    async promptUser() {
        const isCandidate = await this.tasExperimentService?.getTreatment('CESSurvey');
        if (!isCandidate) {
            this.skipSurvey();
            return;
        }
        const sendTelemetry = (userReaction) => {
            this.telemetryService.publicLog('cesSurvey:popup', { userReaction });
        };
        const message = (await this.tasExperimentService?.getTreatment('CESSurveyMessage')) ?? ( localizeWithPath(
            'vs/workbench/contrib/surveys/browser/ces.contribution',
            'cesSurveyQuestion',
            'Got a moment to help the VS Code team? Please tell us about your experience with VS Code so far.'
        ));
        const button = (await this.tasExperimentService?.getTreatment('CESSurveyButton')) ?? ( localizeWithPath(
            'vs/workbench/contrib/surveys/browser/ces.contribution',
            'giveFeedback',
            "Give Feedback"
        ));
        const notification = this.notificationService.prompt(Severity.Info, message, [{
                label: button,
                run: () => {
                    sendTelemetry('accept');
                    let surveyUrl = `${this.productService.cesSurveyUrl}?o=${encodeURIComponent(platform)}&v=${encodeURIComponent(this.productService.version)}&m=${encodeURIComponent(this.telemetryService.machineId)}`;
                    const usedParams = this.productService.surveys
                        ?.filter(surveyData => surveyData.surveyId && surveyData.languageId)
                        .filter(surveyData => this.storageService.getNumber(`${surveyData.surveyId}.editedCount`, -1 , 0) > 0)
                        .map(surveyData => `${encodeURIComponent(surveyData.languageId)}Lang=1`)
                        .join('&');
                    if (usedParams) {
                        surveyUrl += `&${usedParams}`;
                    }
                    this.openerService.open(( URI.parse(surveyUrl)));
                    this.skipSurvey();
                }
            }, {
                label: ( localizeWithPath(
                    'vs/workbench/contrib/surveys/browser/ces.contribution',
                    'remindLater',
                    "Remind Me Later"
                )),
                run: () => {
                    sendTelemetry('remindLater');
                    this.storageService.store(REMIND_LATER_DATE_KEY, ( new Date()).toUTCString(), -1 , 0 );
                    this.schedulePrompt();
                }
            }], {
            sticky: true,
            onCancel: () => {
                sendTelemetry('cancelled');
                this.skipSurvey();
            }
        });
        await Event.toPromise(notification.onDidClose);
    }
    async schedulePrompt() {
        let waitTimeToShowSurvey = 0;
        const remindLaterDate = this.storageService.get(REMIND_LATER_DATE_KEY, -1 , '');
        if (remindLaterDate) {
            const timeToRemind = ( new Date(remindLaterDate)).getTime() + REMIND_LATER_DELAY - Date.now();
            if (timeToRemind > 0) {
                waitTimeToShowSurvey = timeToRemind;
            }
        }
        else {
            const timeFromInstall = Date.now() - ( new Date(this.telemetryService.firstSessionDate)).getTime();
            const isNewInstall = !isNaN(timeFromInstall) && timeFromInstall < MAX_INSTALL_AGE;
            if (!isNewInstall) {
                this.skipSurvey();
                return;
            }
            if (timeFromInstall < WAIT_TIME_TO_SHOW_SURVEY) {
                waitTimeToShowSurvey = WAIT_TIME_TO_SHOW_SURVEY - timeFromInstall;
            }
        }
        this.telemetryService.publicLog('cesSurvey:schedule');
        this.promptDelayer.trigger(async () => {
            await this.promptUser();
        }, Math.max(waitTimeToShowSurvey, MIN_WAIT_TIME_TO_SHOW_SURVEY));
    }
    skipSurvey() {
        this.storageService.store(SKIP_SURVEY_KEY, this.productService.version, -1 , 0 );
    }
};
CESContribution = ( __decorate([
    ( __param(0, IStorageService)),
    ( __param(1, INotificationService)),
    ( __param(2, ITelemetryService)),
    ( __param(3, IOpenerService)),
    ( __param(4, IProductService)),
    ( __param(5, IWorkbenchAssignmentService))
], CESContribution));
if (language === 'en') {
    const workbenchRegistry = ( Registry.as(Extensions.Workbench));
    workbenchRegistry.registerWorkbenchContribution(CESContribution, 3 );
}
