import { Disposable } from '../../../../../base/common/lifecycle.js';
import { localizeWithPath } from '../../../../../nls.js';
import { IKeybindingService } from '../../../../../platform/keybinding/common/keybinding.js';
import { IAccessibleViewService } from '../../../accessibility/browser/accessibleView.js';
import { AccessibilityHelpAction } from '../../../accessibility/browser/accessibleViewActions.js';
import { ITerminalService } from '../../../terminal/browser/terminal.js';
import { TerminalChatContextKeys } from './terminalChat.js';
import { TerminalChatController } from './terminalChatController.js';
class TerminalChatAccessibilityHelpContribution extends Disposable {
    static { this.ID = 'terminalChatAccessiblityHelp'; }
    constructor() {
        super();
        this._register(AccessibilityHelpAction.addImplementation(110, 'terminalChat', runAccessibilityHelpAction, TerminalChatContextKeys.focused));
    }
}
async function runAccessibilityHelpAction(accessor) {
    const accessibleViewService = accessor.get(IAccessibleViewService);
    const terminalService = accessor.get(ITerminalService);
    const instance = terminalService.activeInstance;
    if (!instance) {
        return;
    }
    const helpText = getAccessibilityHelpText(accessor);
    accessibleViewService.show({
        id: "terminal-chat" ,
        verbositySettingKey: "accessibility.verbosity.terminalChat" ,
        provideContent: () => helpText,
        onClose: () => TerminalChatController.get(instance)?.focus(),
        options: { type: "help"  }
    });
}
function getAccessibilityHelpText(accessor) {
    const keybindingService = accessor.get(IKeybindingService);
    const content = [];
    const openAccessibleViewKeybinding = keybindingService.lookupKeybinding('editor.action.accessibleView')?.getAriaLabel();
    const runCommandKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.runCommand" )?.getAriaLabel();
    const insertCommandKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.insertCommand" )?.getAriaLabel();
    const makeRequestKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.makeRequest" )?.getAriaLabel();
    const startChatKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.start" )?.getAriaLabel();
    const focusResponseKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.focusResponse" )?.getAriaLabel();
    const focusInputKeybinding = keybindingService.lookupKeybinding("workbench.action.terminal.chat.focusInput" )?.getAriaLabel();
    content.push(( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.overview',
        "Inline chat occurs within a terminal. It is useful for suggesting terminal commands. Keep in mind that AI generated code may be incorrect."
    )));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.access',
        "It can be activated using the command: Terminal: Start Chat ({0}), which will focus the input box.",
        startChatKeybinding
    )));
    content.push(makeRequestKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.input',
        "The input box is where the user can type a request and can make the request ({0}). The widget will be closed and all content will be discarded when the Escape key is pressed and the terminal will regain focus.",
        makeRequestKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.inputNoKb',
        "The input box is where the user can type a request and can make the request by tabbing to the Make Request button, which is not currently triggerable via keybindings. The widget will be closed and all content will be discarded when the Escape key is pressed and the terminal will regain focus."
    )));
    content.push(openAccessibleViewKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.inspectResponseMessage',
        'The response can be inspected in the accessible view ({0}).',
        openAccessibleViewKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.inspectResponseNoKb',
        'With the input box focused, inspect the response in the accessible view via the Open Accessible View command, which is currently not triggerable by a keybinding.'
    )));
    content.push(focusResponseKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.focusResponse',
        'Reach the response from the input box ({0}).',
        focusResponseKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.focusResponseNoKb',
        'Reach the response from the input box by tabbing or assigning a keybinding for the command: Focus Terminal Response.'
    )));
    content.push(focusInputKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.focusInput',
        'Reach the input box from the response ({0}).',
        focusInputKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.focusInputNoKb',
        'Reach the response from the input box by shift+tabbing or assigning a keybinding for the command: Focus Terminal Input.'
    )));
    content.push(runCommandKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.runCommand',
        'With focus in the input box or command editor, the Terminal: Run Chat Command ({0}) action.',
        runCommandKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.runCommandNoKb',
        'Run a command by tabbing to the button as the action is currently not triggerable by a keybinding.'
    )));
    content.push(insertCommandKeybinding ? ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.insertCommand',
        'With focus in the input box command editor, the Terminal: Insert Chat Command ({0}) action.',
        insertCommandKeybinding
    )) : ( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.insertCommandNoKb',
        'Insert a command by tabbing to the button as the action is currently not triggerable by a keybinding.'
    )));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'inlineChat.toolbar',
        "Use tab to reach conditional parts like commands, status, message responses and more."
    )));
    content.push(( localizeWithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatAccessibilityHelp',
        'chat.signals',
        "Accessibility Signals can be changed via settings with a prefix of signals.chat. By default, if a request takes more than 4 seconds, you will hear a sound indicating that progress is still occurring."
    )));
    return content.join('\n\n');
}
export { TerminalChatAccessibilityHelpContribution, getAccessibilityHelpText, runAccessibilityHelpAction };
