import { __decorate, __param } from '../../../../../../../../external/tslib/tslib.es6.js';
import { Emitter } from '../../../../../base/common/event.js';
import { Disposable } from '../../../../../base/common/lifecycle.js';
import { localizeWithPath } from '../../../../../nls.js';
import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
import { TerminalLink } from './terminalLink.js';
let TerminalLinkDetectorAdapter = class TerminalLinkDetectorAdapter extends Disposable {
    constructor(_detector, _instantiationService) {
        super();
        this._detector = _detector;
        this._instantiationService = _instantiationService;
        this._onDidActivateLink = this._register(( new Emitter()));
        this.onDidActivateLink = this._onDidActivateLink.event;
        this._onDidShowHover = this._register(( new Emitter()));
        this.onDidShowHover = this._onDidShowHover.event;
        this._activeProvideLinkRequests = ( new Map());
    }
    async provideLinks(bufferLineNumber, callback) {
        let activeRequest = this._activeProvideLinkRequests.get(bufferLineNumber);
        if (activeRequest) {
            await activeRequest;
            callback(this._activeLinks);
            return;
        }
        if (this._activeLinks) {
            for (const link of this._activeLinks) {
                link.dispose();
            }
        }
        activeRequest = this._provideLinks(bufferLineNumber);
        this._activeProvideLinkRequests.set(bufferLineNumber, activeRequest);
        this._activeLinks = await activeRequest;
        this._activeProvideLinkRequests.delete(bufferLineNumber);
        callback(this._activeLinks);
    }
    async _provideLinks(bufferLineNumber) {
        const links = [];
        let startLine = bufferLineNumber - 1;
        let endLine = startLine;
        const lines = [
            this._detector.xterm.buffer.active.getLine(startLine)
        ];
        const maxLineContext = Math.max(this._detector.maxLinkLength, this._detector.xterm.cols);
        const minStartLine = Math.max(startLine - maxLineContext, 0);
        const maxEndLine = Math.min(endLine + maxLineContext, this._detector.xterm.buffer.active.length);
        while (startLine >= minStartLine && this._detector.xterm.buffer.active.getLine(startLine)?.isWrapped) {
            lines.unshift(this._detector.xterm.buffer.active.getLine(startLine - 1));
            startLine--;
        }
        while (endLine < maxEndLine && this._detector.xterm.buffer.active.getLine(endLine + 1)?.isWrapped) {
            lines.push(this._detector.xterm.buffer.active.getLine(endLine + 1));
            endLine++;
        }
        const detectedLinks = await this._detector.detect(lines, startLine, endLine);
        for (const link of detectedLinks) {
            links.push(this._createTerminalLink(link, async (event) => this._onDidActivateLink.fire({ link, event })));
        }
        return links;
    }
    _createTerminalLink(l, activateCallback) {
        if (!l.disableTrimColon && l.text.length > 0 && l.text.charAt(l.text.length - 1) === ':') {
            l.text = l.text.slice(0, -1);
            l.bufferRange.end.x--;
        }
        return this._instantiationService.createInstance(TerminalLink, this._detector.xterm, l.bufferRange, l.text, l.uri, l.parsedLink, l.actions, this._detector.xterm.buffer.active.viewportY, activateCallback, (link, viewportRange, modifierDownCallback, modifierUpCallback) => this._onDidShowHover.fire({
            link,
            viewportRange,
            modifierDownCallback,
            modifierUpCallback
        }), l.type !== "Search" ,
        l.label || this._getLabel(l.type), l.type);
    }
    _getLabel(type) {
        switch (type) {
            case "Search" : return ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/links/browser/terminalLinkDetectorAdapter',
                'searchWorkspace',
                'Search workspace'
            ));
            case "LocalFile" : return ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/links/browser/terminalLinkDetectorAdapter',
                'openFile',
                'Open file in editor'
            ));
            case "LocalFolderInWorkspace" : return ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/links/browser/terminalLinkDetectorAdapter',
                'focusFolder',
                'Focus folder in explorer'
            ));
            case "LocalFolderOutsideWorkspace" : return ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/links/browser/terminalLinkDetectorAdapter',
                'openFolder',
                'Open folder in new window'
            ));
            case "Url" :
            default:
                return ( localizeWithPath(
                    'vs/workbench/contrib/terminalContrib/links/browser/terminalLinkDetectorAdapter',
                    'followLink',
                    'Follow link'
                ));
        }
    }
};
TerminalLinkDetectorAdapter = ( __decorate([
    ( __param(1, IInstantiationService))
], TerminalLinkDetectorAdapter));
export { TerminalLinkDetectorAdapter };
