import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { Disposable, DisposableStore } from '../../../../base/common/lifecycle.js';
import { localizeWithPath } from '../../../../nls.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { IViewsService } from '../../../services/views/common/viewsService.js';
import { getTestingConfiguration } from '../common/configuration.js';
import { isFailedState } from '../common/testingStates.js';
import { ITestResultService } from '../common/testResultService.js';
let TestingProgressTrigger = class TestingProgressTrigger extends Disposable {
    constructor(resultService, configurationService, viewsService) {
        super();
        this.configurationService = configurationService;
        this.viewsService = viewsService;
        this._register(resultService.onResultsChanged((e) => {
            if ('started' in e) {
                this.attachAutoOpenForNewResults(e.started);
            }
        }));
    }
    attachAutoOpenForNewResults(result) {
        if (result.request.isUiTriggered === false) {
            return;
        }
        const cfg = getTestingConfiguration(this.configurationService, "testing.openTesting" );
        if (cfg === "neverOpen" ) {
            return;
        }
        if (cfg === "openExplorerOnTestStart" ) {
            return this.openExplorerView();
        }
        if (cfg === "openOnTestStart" ) {
            return this.openResultsView();
        }
        const disposable = ( new DisposableStore());
        disposable.add(result.onComplete(() => disposable.dispose()));
        disposable.add(result.onChange(e => {
            if (e.reason === 1  && isFailedState(e.item.ownComputedState)) {
                this.openResultsView();
                disposable.dispose();
            }
        }));
    }
    openExplorerView() {
        this.viewsService.openView("workbench.view.testing" , false);
    }
    openResultsView() {
        this.viewsService.openView("workbench.panel.testResults.view" , false);
    }
};
TestingProgressTrigger = ( __decorate([
    ( __param(0, ITestResultService)),
    ( __param(1, IConfigurationService)),
    ( __param(2, IViewsService))
], TestingProgressTrigger));
const collectTestStateCounts = (isRunning, results) => {
    let passed = 0;
    let failed = 0;
    let skipped = 0;
    let running = 0;
    let queued = 0;
    for (const result of results) {
        const count = result.counts;
        failed += count[6 ] + count[4 ];
        passed += count[3 ];
        skipped += count[5 ];
        running += count[2 ];
        queued += count[1 ];
    }
    return {
        isRunning,
        passed,
        failed,
        runSoFar: passed + failed,
        totalWillBeRun: passed + failed + queued + running,
        skipped,
    };
};
const getTestProgressText = ({ isRunning, passed, runSoFar, totalWillBeRun, skipped, failed }) => {
    let percent = passed / runSoFar * 100;
    if (failed > 0) {
        percent = Math.min(percent, 99.9);
    }
    else if (runSoFar === 0) {
        percent = 0;
    }
    if (isRunning) {
        if (runSoFar === 0) {
            return ( localizeWithPath(
                'vs/workbench/contrib/testing/browser/testingProgressUiService',
                'testProgress.runningInitial',
                'Running tests...'
            ));
        }
        else if (skipped === 0) {
            return ( localizeWithPath(
                'vs/workbench/contrib/testing/browser/testingProgressUiService',
                'testProgress.running',
                'Running tests, {0}/{1} passed ({2}%)',
                passed,
                totalWillBeRun,
                percent.toPrecision(3)
            ));
        }
        else {
            return ( localizeWithPath(
                'vs/workbench/contrib/testing/browser/testingProgressUiService',
                'testProgressWithSkip.running',
                'Running tests, {0}/{1} tests passed ({2}%, {3} skipped)',
                passed,
                totalWillBeRun,
                percent.toPrecision(3),
                skipped
            ));
        }
    }
    else {
        if (skipped === 0) {
            return ( localizeWithPath(
                'vs/workbench/contrib/testing/browser/testingProgressUiService',
                'testProgress.completed',
                '{0}/{1} tests passed ({2}%)',
                passed,
                runSoFar,
                percent.toPrecision(3)
            ));
        }
        else {
            return ( localizeWithPath(
                'vs/workbench/contrib/testing/browser/testingProgressUiService',
                'testProgressWithSkip.completed',
                '{0}/{1} tests passed ({2}%, {3} skipped)',
                passed,
                runSoFar,
                percent.toPrecision(3),
                skipped
            ));
        }
    }
};
export { TestingProgressTrigger, collectTestStateCounts, getTestProgressText };
