import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { groupBy } from '../../../../base/common/arrays.js';
import { CancellationToken, CancellationTokenSource } from '../../../../base/common/cancellation.js';
import { Emitter } from '../../../../base/common/event.js';
import { Iterable } from '../../../../base/common/iterator.js';
import { Disposable, toDisposable, DisposableStore } from '../../../../base/common/lifecycle.js';
import { localizeWithPath } from '../../../../nls.js';
import { IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { INotificationService } from '../../../../platform/notification/common/notification.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { IWorkspaceTrustRequestService } from '../../../../platform/workspace/common/workspaceTrust.js';
import { MainThreadTestCollection } from './mainThreadTestCollection.js';
import { MutableObservableValue } from './observableValue.js';
import { StoredValue } from './storedValue.js';
import { TestExclusions } from './testExclusions.js';
import { TestId } from './testId.js';
import { TestingContextKeys } from './testingContextKeys.js';
import { ITestProfileService, canUseProfileWithTest } from './testProfileService.js';
import { ITestResultService } from './testResultService.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { getTestingConfiguration } from './configuration.js';
import { isDefined } from '../../../../base/common/types.js';
import { IUriIdentityService } from '../../../../platform/uriIdentity/common/uriIdentity.js';
let TestService = class TestService extends Disposable {
    constructor(contextKeyService, instantiationService, uriIdentityService, storage, editorService, testProfiles, notificationService, configurationService, testResults, workspaceTrustRequestService) {
        super();
        this.uriIdentityService = uriIdentityService;
        this.storage = storage;
        this.editorService = editorService;
        this.testProfiles = testProfiles;
        this.notificationService = notificationService;
        this.configurationService = configurationService;
        this.testResults = testResults;
        this.workspaceTrustRequestService = workspaceTrustRequestService;
        this.testControllers = ( new Map());
        this.cancelExtensionTestRunEmitter = ( new Emitter());
        this.willProcessDiffEmitter = ( new Emitter());
        this.didProcessDiffEmitter = ( new Emitter());
        this.testRefreshCancellations = ( new Set());
        this.uiRunningTests = ( new Map());
        this.onWillProcessDiff = this.willProcessDiffEmitter.event;
        this.onDidProcessDiff = this.didProcessDiffEmitter.event;
        this.onDidCancelTestRun = this.cancelExtensionTestRunEmitter.event;
        this.collection = ( new MainThreadTestCollection(this.uriIdentityService, this.expandTest.bind(this)));
        this.showInlineOutput = MutableObservableValue.stored(this._register(( new StoredValue({
            key: 'inlineTestOutputVisible',
            scope: 1 ,
            target: 0
        }, this.storage))), true);
        this.excluded = instantiationService.createInstance(TestExclusions);
        this.providerCount = TestingContextKeys.providerCount.bindTo(contextKeyService);
        this.canRefreshTests = TestingContextKeys.canRefreshTests.bindTo(contextKeyService);
        this.isRefreshingTests = TestingContextKeys.isRefreshingTests.bindTo(contextKeyService);
        this.activeEditorHasTests = TestingContextKeys.activeEditorHasTests.bindTo(contextKeyService);
        this._register(editorService.onDidActiveEditorChange(() => this.updateEditorContextKeys()));
    }
    async expandTest(id, levels) {
        await this.testControllers.get(TestId.fromString(id).controllerId)?.expandTest(id, levels);
    }
    cancelTestRun(runId) {
        this.cancelExtensionTestRunEmitter.fire({ runId });
        if (runId === undefined) {
            for (const runCts of ( this.uiRunningTests.values())) {
                runCts.cancel();
            }
        }
        else {
            this.uiRunningTests.get(runId)?.cancel();
        }
    }
    async runTests(req, token = CancellationToken.None) {
        const resolved = {
            targets: [],
            exclude: req.exclude?.map(t => t.item.extId),
            continuous: req.continuous,
        };
        for (const profile of this.testProfiles.getGroupDefaultProfiles(req.group)) {
            const testIds = ( req.tests.filter(t => canUseProfileWithTest(profile, t)).map(t => t.item.extId));
            if (testIds.length) {
                resolved.targets.push({
                    testIds: testIds,
                    profileGroup: profile.group,
                    profileId: profile.profileId,
                    controllerId: profile.controllerId,
                });
            }
        }
        if (resolved.targets.length === 0) {
            for (const byController of groupBy(req.tests, (a, b) => a.controllerId === b.controllerId ? 0 : 1)) {
                const profiles = this.testProfiles.getControllerProfiles(byController[0].controllerId);
                const withControllers = ( byController.map(test => ({
                    profile: profiles.find(p => p.group === req.group && canUseProfileWithTest(p, test)),
                    test,
                })));
                for (const byProfile of groupBy(withControllers, (a, b) => a.profile === b.profile ? 0 : 1)) {
                    const profile = byProfile[0].profile;
                    if (profile) {
                        resolved.targets.push({
                            testIds: ( byProfile.map(t => t.test.item.extId)),
                            profileGroup: req.group,
                            profileId: profile.profileId,
                            controllerId: profile.controllerId,
                        });
                    }
                }
            }
        }
        return this.runResolvedTests(resolved, token);
    }
    async startContinuousRun(req, token) {
        if (!req.exclude) {
            req.exclude = [...this.excluded.all];
        }
        const trust = await this.workspaceTrustRequestService.requestWorkspaceTrust({
            message: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/testServiceImpl',
                'testTrust',
                "Running tests may execute code in your workspace."
            )),
        });
        if (!trust) {
            return;
        }
        const byController = groupBy(req.targets, (a, b) => a.controllerId.localeCompare(b.controllerId));
        const requests = ( byController.map(
            group => this.testControllers.get(group[0].controllerId)?.startContinuousRun(( group.map(controlReq => ({
                excludeExtIds: req.exclude.filter(t => !controlReq.testIds.includes(t)),
                profileId: controlReq.profileId,
                controllerId: controlReq.controllerId,
                testIds: controlReq.testIds,
            }))), token).then(result => {
                const errs = ( result.map(r => r.error)).filter(isDefined);
                if (errs.length) {
                    this.notificationService.error(( localizeWithPath(
                        'vs/workbench/contrib/testing/common/testServiceImpl',
                        'testError',
                        'An error occurred attempting to run tests: {0}',
                        errs.join(' ')
                    )));
                }
            })
        ));
        await Promise.all(requests);
    }
    async runResolvedTests(req, token = CancellationToken.None) {
        if (!req.exclude) {
            req.exclude = [...this.excluded.all];
        }
        const result = this.testResults.createLiveResult(req);
        const trust = await this.workspaceTrustRequestService.requestWorkspaceTrust({
            message: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/testServiceImpl',
                'testTrust',
                "Running tests may execute code in your workspace."
            )),
        });
        if (!trust) {
            result.markComplete();
            return result;
        }
        try {
            const cancelSource = ( new CancellationTokenSource(token));
            this.uiRunningTests.set(result.id, cancelSource);
            const byController = groupBy(req.targets, (a, b) => a.controllerId.localeCompare(b.controllerId));
            const requests = ( byController.map(
                group => this.testControllers.get(group[0].controllerId)?.runTests(( group.map(controlReq => ({
                    runId: result.id,
                    excludeExtIds: req.exclude.filter(t => !controlReq.testIds.includes(t)),
                    profileId: controlReq.profileId,
                    controllerId: controlReq.controllerId,
                    testIds: controlReq.testIds,
                }))), cancelSource.token).then(result => {
                    const errs = ( result.map(r => r.error)).filter(isDefined);
                    if (errs.length) {
                        this.notificationService.error(( localizeWithPath(
                            'vs/workbench/contrib/testing/common/testServiceImpl',
                            'testError',
                            'An error occurred attempting to run tests: {0}',
                            errs.join(' ')
                        )));
                    }
                })
            ));
            await this.saveAllBeforeTest(req);
            await Promise.all(requests);
            return result;
        }
        finally {
            this.uiRunningTests.delete(result.id);
            result.markComplete();
        }
    }
    publishDiff(_controllerId, diff) {
        this.willProcessDiffEmitter.fire(diff);
        this.collection.apply(diff);
        this.updateEditorContextKeys();
        this.didProcessDiffEmitter.fire(diff);
    }
    getTestController(id) {
        return this.testControllers.get(id);
    }
    async syncTests() {
        const cts = ( new CancellationTokenSource());
        try {
            await Promise.all(( [...( this.testControllers.values())].map(c => c.syncTests(cts.token))));
        }
        finally {
            cts.dispose(true);
        }
    }
    async refreshTests(controllerId) {
        const cts = ( new CancellationTokenSource());
        this.testRefreshCancellations.add(cts);
        this.isRefreshingTests.set(true);
        try {
            if (controllerId) {
                await this.testControllers.get(controllerId)?.refreshTests(cts.token);
            }
            else {
                await Promise.all(( [...( this.testControllers.values())].map(c => c.refreshTests(cts.token))));
            }
        }
        finally {
            this.testRefreshCancellations.delete(cts);
            this.isRefreshingTests.set(this.testRefreshCancellations.size > 0);
            cts.dispose(true);
        }
    }
    cancelRefreshTests() {
        for (const cts of this.testRefreshCancellations) {
            cts.cancel();
        }
        this.testRefreshCancellations.clear();
        this.isRefreshingTests.set(false);
    }
    registerTestController(id, controller) {
        this.testControllers.set(id, controller);
        this.providerCount.set(this.testControllers.size);
        this.updateCanRefresh();
        const disposable = ( new DisposableStore());
        disposable.add(toDisposable(() => {
            const diff = [];
            for (const root of this.collection.rootItems) {
                if (root.controllerId === id) {
                    diff.push({ op: 3 , itemId: root.item.extId });
                }
            }
            this.publishDiff(id, diff);
            if (this.testControllers.delete(id)) {
                this.providerCount.set(this.testControllers.size);
                this.updateCanRefresh();
            }
        }));
        disposable.add(controller.canRefresh.onDidChange(this.updateCanRefresh, this));
        return disposable;
    }
    updateEditorContextKeys() {
        const uri = this.editorService.activeEditor?.resource;
        if (uri) {
            this.activeEditorHasTests.set(!Iterable.isEmpty(this.collection.getNodeByUrl(uri)));
        }
        else {
            this.activeEditorHasTests.set(false);
        }
    }
    async saveAllBeforeTest(req, configurationService = this.configurationService, editorService = this.editorService) {
        if (req.isUiTriggered === false) {
            return;
        }
        const saveBeforeTest = getTestingConfiguration(this.configurationService, "testing.saveBeforeTest" );
        if (saveBeforeTest) {
            await editorService.saveAll();
        }
        return;
    }
    updateCanRefresh() {
        this.canRefreshTests.set(( Iterable.some(( this.testControllers.values()), t => t.canRefresh.value)));
    }
};
TestService = ( __decorate([
    ( __param(0, IContextKeyService)),
    ( __param(1, IInstantiationService)),
    ( __param(2, IUriIdentityService)),
    ( __param(3, IStorageService)),
    ( __param(4, IEditorService)),
    ( __param(5, ITestProfileService)),
    ( __param(6, INotificationService)),
    ( __param(7, IConfigurationService)),
    ( __param(8, ITestResultService)),
    ( __param(9, IWorkspaceTrustRequestService))
], TestService));
export { TestService };
