import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { windowOpenNoOpener } from '../../../../base/browser/dom.js';
import { CancellationToken } from '../../../../base/common/cancellation.js';
import { toDisposable } from '../../../../base/common/lifecycle.js';
import { userAgent } from '../../../../base/common/platform.js';
import { normalizeGitHubUrl } from '../../../../platform/issue/common/issueReporterUtil.js';
import { ILogService } from '../../../../platform/log/common/log.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { IExtensionService } from '../../extensions/common/extensions.js';
let WebIssueService = class WebIssueService {
    constructor(extensionService, productService, logService) {
        this.extensionService = extensionService;
        this.productService = productService;
        this.logService = logService;
        this._handlers = ( new Map());
        this._providers = ( new Map());
    }
    async openProcessExplorer() {
        console.error('openProcessExplorer is not implemented in web');
    }
    async openReporter(options) {
        const extensionId = options.extensionId;
        if (!extensionId) {
            if (this.productService.reportIssueUrl) {
                const uri = this.getIssueUriFromStaticContent(this.productService.reportIssueUrl);
                windowOpenNoOpener(uri);
                return;
            }
            throw new Error(`No issue reporting URL configured for ${this.productService.nameLong}.`);
        }
        if (( this._handlers.has(extensionId))) {
            try {
                const uri = await this.getIssueUriFromHandler(extensionId, CancellationToken.None);
                windowOpenNoOpener(uri);
                return;
            }
            catch (e) {
                this.logService.error(e);
            }
        }
        const selectedExtension = this.extensionService.extensions.filter(ext => ext.identifier.value === options.extensionId)[0];
        const extensionGitHubUrl = this.getExtensionGitHubUrl(selectedExtension);
        if (!extensionGitHubUrl) {
            throw new Error(`Unable to find issue reporting url for ${extensionId}`);
        }
        const uri = this.getIssueUriFromStaticContent(`${extensionGitHubUrl}/issues/new`, selectedExtension);
        windowOpenNoOpener(uri);
    }
    registerIssueUriRequestHandler(extensionId, handler) {
        this._handlers.set(extensionId, handler);
        return toDisposable(() => this._handlers.delete(extensionId));
    }
    registerIssueDataProvider(extensionId, handler) {
        this._providers.set(extensionId, handler);
        return toDisposable(() => this._providers.delete(extensionId));
    }
    async getIssueUriFromHandler(extensionId, token) {
        const handler = this._handlers.get(extensionId);
        if (!handler) {
            throw new Error(`No handler registered for extension ${extensionId}`);
        }
        const result = await handler.provideIssueUrl(token);
        return ( result.toString(true));
    }
    getExtensionGitHubUrl(extension) {
        if (extension.isBuiltin && this.productService.reportIssueUrl) {
            return normalizeGitHubUrl(this.productService.reportIssueUrl);
        }
        let repositoryUrl = '';
        const bugsUrl = extension?.bugs?.url;
        const extensionUrl = extension?.repository?.url;
        if (bugsUrl && bugsUrl.match(/^https?:\/\/github\.com\/(.*)/)) {
            repositoryUrl = normalizeGitHubUrl(bugsUrl);
        }
        else if (extensionUrl && extensionUrl.match(/^https?:\/\/github\.com\/(.*)/)) {
            repositoryUrl = normalizeGitHubUrl(extensionUrl);
        }
        return repositoryUrl;
    }
    getIssueUriFromStaticContent(baseUri, extension) {
        const issueDescription = `ADD ISSUE DESCRIPTION HERE

Version: ${this.productService.version}
Commit: ${this.productService.commit ?? 'unknown'}
User Agent: ${userAgent ?? 'unknown'}
Embedder: ${this.productService.embedderIdentifier ?? 'unknown'}
${extension?.version ? `\nExtension version: ${extension.version}` : ''}
<!-- generated by web issue reporter -->`;
        return `${baseUri}?body=${encodeURIComponent(issueDescription)}&labels=web`;
    }
};
WebIssueService = ( __decorate([
    ( __param(0, IExtensionService)),
    ( __param(1, IProductService)),
    ( __param(2, ILogService))
], WebIssueService));
export { WebIssueService };
